package com.mortals.xhx.module.area.service.impl;

import com.alibaba.fastjson.JSON;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.feign.area.IApiAreaFeign;
import com.mortals.xhx.module.area.model.AreaEntity;
import com.mortals.xhx.module.area.model.AreaQuery;
import com.mortals.xhx.module.area.model.AreaTreeSelect;
import com.mortals.xhx.feign.rsp.ApiResp;
import com.mortals.xhx.module.area.dao.AreaDao;
import com.mortals.xhx.module.area.service.AreaService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * AreaService
 * 区域 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("areaService")
@Slf4j
public class AreaServiceImpl extends AbstractCRUDServiceImpl<AreaDao, AreaEntity, Long> implements AreaService {

    @Autowired
    private IApiAreaFeign apiAreaFeign;


    @Override
    public String getListByParentId(String parentId, Context context) {
        if (ObjectUtils.isEmpty(parentId)) {
            parentId = "0";
        }
        String resp = apiAreaFeign.getListByParentId(parentId);

        ApiResp apiResp = JSON.parseObject(resp, ApiResp.class);
        if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
            throw new AppException("获取区域数据失败：" + apiResp.getMsg());
        }
        log.info("getListByParentId=>{}", resp);
        return resp;

    }

    @Override
    public String viewArea(Long id, Context context) {
        String resp = apiAreaFeign.viewAreaInfo(id);
        ApiResp apiResp = JSON.parseObject(resp, ApiResp.class);
        if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
            throw new AppException("获取区域数据失败：" + apiResp.getMsg());
        }
        log.info("viewArea=>{}", resp);
        return resp;
    }

    @Override
    public String list(AreaEntity query, Context context) {
        log.info("areaListReq==》{}", JSON.toJSONString(query));
        String resp = apiAreaFeign.list(query);
        ApiResp apiResp = JSON.parseObject(resp, ApiResp.class);
        if (apiResp.getCode() != YesNoEnum.YES.getValue()) {
            throw new AppException("获取区域列表数据失败：" + apiResp.getMsg());
        }
        log.info("listAreaResp=>{}", resp);
        return resp;
    }


    @Override
    public boolean hasChildByAreaId(String areaId) {
        List<AreaEntity> list = this.find(new AreaQuery().pid(areaId));
        return list.size() > 0 ? true : false;
    }

    @Override
    public List<AreaEntity> selectAreaList(AreaEntity area) {
        return this.find(new AreaQuery());
    }

    @Override
    public List<AreaTreeSelect> buildAreaTreeSelect(List<AreaEntity> list) {
        List<AreaEntity> returnList = new ArrayList<>();
        List<Long> tempList = list.stream().map(AreaEntity::getId).collect(Collectors.toList());
        for (AreaEntity areaEntity : list) {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(areaEntity.getPid())) {
                recursionFn(list, areaEntity);
                returnList.add(areaEntity);
            }
        }
        if (returnList.isEmpty()) {
            returnList = list;
        }

        return returnList.stream().map(AreaTreeSelect::new).collect(Collectors.toList());
    }


    /**
     * 递归列表
     */
    private void recursionFn(List<AreaEntity> list, AreaEntity t) {
        // 得到子节点列表
        List<AreaEntity> childList = getChildList(list, t);
        t.setChildren(childList);
        for (AreaEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<AreaEntity> getChildList(List<AreaEntity> list, AreaEntity t) {
        return list.stream().map(item -> {
            if (!ObjectUtils.isEmpty(item.getPid()) && item.getPid() == t.getIid()) {
                return item;
            }
            return null;
        }).filter(f -> f != null).collect(Collectors.toList());
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<AreaEntity> list, AreaEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }


}